'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ReactMeteorData = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /* global Package */
/* eslint-disable react/prefer-stateless-function */

exports.default = connect;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _meteor = require('meteor/meteor');

var _tracker = require('meteor/tracker');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// A class to keep the state and utility methods needed to manage
// the Meteor data for a component.
var MeteorDataManager = function () {
    function MeteorDataManager(component) {
        _classCallCheck(this, MeteorDataManager);

        this.component = component;
        this.computation = null;
        this.oldData = null;
    }

    _createClass(MeteorDataManager, [{
        key: 'dispose',
        value: function dispose() {
            if (this.computation) {
                this.computation.stop();
                this.computation = null;
            }
        }
    }, {
        key: 'calculateData',
        value: function calculateData() {
            var component = this.component;

            if (!component.getMeteorData) {
                return null;
            }

            // When rendering on the server, we don't want to use the Tracker.
            // We only do the first rendering on the server so we can get the data right away
            if (_meteor.Meteor.isServer) {
                return component.getMeteorData();
            }

            if (this.computation) {
                this.computation.stop();
                this.computation = null;
            }

            var data = void 0;
            // Use Tracker.nonreactive in case we are inside a Tracker Computation.
            // This can happen if someone calls `ReactDOM.render` inside a Computation.
            // In that case, we want to opt out of the normal behavior of nested
            // Computations, where if the outer one is invalidated or stopped,
            // it stops the inner one.
            this.computation = _tracker.Tracker.nonreactive(function () {
                return _tracker.Tracker.autorun(function (c) {
                    if (c.firstRun) {
                        var savedSetState = component.setState;
                        try {
                            component.setState = function () {
                                throw new Error('Can\'t call `setState` inside `getMeteorData` as this could ' + 'cause an endless loop. To respond to Meteor data changing, ' + 'consider making this component a \"wrapper component\" that ' + 'only fetches data and passes it in as props to a child ' + 'component. Then you can use `componentWillReceiveProps` in ' + 'that child component.');
                            };

                            data = component.getMeteorData();
                        } finally {
                            component.setState = savedSetState;
                        }
                    } else {
                        // Stop this computation instead of using the re-run.
                        // We use a brand-new autorun for each call to getMeteorData
                        // to capture dependencies on any reactive data sources that
                        // are accessed.  The reason we can't use a single autorun
                        // for the lifetime of the component is that Tracker only
                        // re-runs autoruns at flush time, while we need to be able to
                        // re-call getMeteorData synchronously whenever we want, e.g.
                        // from componentWillUpdate.
                        c.stop();
                        // Calling forceUpdate() triggers componentWillUpdate which
                        // recalculates getMeteorData() and re-renders the component.
                        component.forceUpdate();
                    }
                });
            });

            if (Package.mongo && Package.mongo.Mongo) {
                Object.keys(data).forEach(function (key) {
                    if (data[key] instanceof Package.mongo.Mongo.Cursor) {
                        console.warn('Warning: you are returning a Mongo cursor from getMeteorData. ' + 'This value will not be reactive. You probably want to call ' + '`.fetch()` on the cursor before returning it.');
                    }
                });
            }

            return data;
        }
    }, {
        key: 'updateData',
        value: function updateData(newData) {
            var component = this.component;
            var oldData = this.oldData;

            if (!(newData && (typeof newData === 'undefined' ? 'undefined' : _typeof(newData)) === 'object')) {
                throw new Error('Expected object returned from getMeteorData');
            }
            // update componentData in place based on newData
            for (var key in newData) {
                component.data[key] = newData[key];
            }
            // if there is oldData (which is every time this method is called
            // except the first), delete keys in newData that aren't in
            // oldData.  don't interfere with other keys, in case we are
            // co-existing with something else that writes to a component's
            // this.data.
            if (oldData) {
                for (var _key in oldData) {
                    if (!(_key in newData)) {
                        delete component.data[_key];
                    }
                }
            }
            this.oldData = newData;
        }
    }]);

    return MeteorDataManager;
}();

var ReactMeteorData = exports.ReactMeteorData = {
    componentWillMount: function componentWillMount() {
        this.data = {};
        this._meteorDataManager = new MeteorDataManager(this);
        var newData = this._meteorDataManager.calculateData();
        this._meteorDataManager.updateData(newData);
    },
    componentWillUpdate: function componentWillUpdate(nextProps, nextState) {
        var saveProps = this.props;
        var saveState = this.state;
        var newData = void 0;
        try {
            // Temporarily assign this.state and this.props,
            // so that they are seen by getMeteorData!
            // This is a simulation of how the proposed Observe API
            // for React will work, which calls observe() after
            // componentWillUpdate and after props and state are
            // updated, but before render() is called.
            // See https://github.com/facebook/react/issues/3398.
            this.props = nextProps;
            this.state = nextState;
            newData = this._meteorDataManager.calculateData();
        } finally {
            this.props = saveProps;
            this.state = saveState;
        }

        this._meteorDataManager.updateData(newData);
    },
    componentWillUnmount: function componentWillUnmount() {
        this._meteorDataManager.dispose();
    }
};

var ReactComponent = function (_React$Component) {
    _inherits(ReactComponent, _React$Component);

    function ReactComponent() {
        _classCallCheck(this, ReactComponent);

        return _possibleConstructorReturn(this, (ReactComponent.__proto__ || Object.getPrototypeOf(ReactComponent)).apply(this, arguments));
    }

    return ReactComponent;
}(_react2.default.Component);

Object.assign(ReactComponent.prototype, ReactMeteorData);

var ReactPureComponent = function (_React$PureComponent) {
    _inherits(ReactPureComponent, _React$PureComponent);

    function ReactPureComponent() {
        _classCallCheck(this, ReactPureComponent);

        return _possibleConstructorReturn(this, (ReactPureComponent.__proto__ || Object.getPrototypeOf(ReactPureComponent)).apply(this, arguments));
    }

    return ReactPureComponent;
}(_react2.default.PureComponent);

Object.assign(ReactPureComponent.prototype, ReactMeteorData);

function connect(options) {
    var expandedOptions = options;
    if (typeof options === 'function') {
        expandedOptions = {
            getMeteorData: options
        };
    }

    var _expandedOptions = expandedOptions,
        _getMeteorData = _expandedOptions.getMeteorData,
        _expandedOptions$pure = _expandedOptions.pure,
        pure = _expandedOptions$pure === undefined ? true : _expandedOptions$pure;


    var BaseComponent = pure ? ReactPureComponent : ReactComponent;
    return function (WrappedComponent) {
        return function (_BaseComponent) {
            _inherits(ReactMeteorDataComponent, _BaseComponent);

            function ReactMeteorDataComponent() {
                _classCallCheck(this, ReactMeteorDataComponent);

                return _possibleConstructorReturn(this, (ReactMeteorDataComponent.__proto__ || Object.getPrototypeOf(ReactMeteorDataComponent)).apply(this, arguments));
            }

            _createClass(ReactMeteorDataComponent, [{
                key: 'getMeteorData',
                value: function getMeteorData() {
                    return _getMeteorData(this.props);
                }
            }, {
                key: 'render',
                value: function render() {
                    return _react2.default.createElement(WrappedComponent, _extends({}, this.props, this.data));
                }
            }]);

            return ReactMeteorDataComponent;
        }(BaseComponent);
    };
}